import sys

__all__ = ['reraise']


def exec_(_code_, _globs_=None, _locs_=None):
    """Execute code in a namespace."""
    if _globs_ is None:
        frame = sys._getframe(1)
        _globs_ = frame.f_globals
        if _locs_ is None:
            _locs_ = frame.f_locals
        del frame
    elif _locs_ is None:
        _locs_ = _globs_
    exec("""exec _code_ in _globs_, _locs_""")

exec_("""def reraise(tp, value, tb=None):
    try:
        raise tp, value, tb
    finally:
        tb = None
""")



from __future__ import print_function, division, absolute_import
from gevent import monkey
monkey.patch_all()
import os

import socket
import gevent.testing as greentest
# Be careful not to have TestTCP as a bare attribute in this module,
# even aliased, to avoid running duplicate tests
from gevent.tests import test__socket
import ssl

from gevent.testing import PY2

def ssl_listener(private_key, certificate):
    raw_listener = socket.socket()
    greentest.bind_and_listen(raw_listener)
    # pylint:disable=deprecated-method
    sock = ssl.wrap_socket(raw_listener, private_key, certificate, server_side=True)
    return sock, raw_listener


class TestSSL(test__socket.TestTCP):

    # To generate:
    # openssl req -x509 -newkey rsa:4096 -keyout test_server.key -out test_server.crt -days 36500 -nodes -subj '/CN=localhost'
    certfile = os.path.join(os.path.dirname(__file__), 'test_server.crt')
    privfile = os.path.join(os.path.dirname(__file__), 'test_server.key')
    # Python 2.x has socket.sslerror (which  is an alias for
    # ssl.SSLError); That's gone in Py3 though. In Python 2, most timeouts are raised
    # as SSLError, but Python 3 raises the normal socket.timeout instead. So this has
    # the effect of making TIMEOUT_ERROR be SSLError on Py2 and socket.timeout on Py3
    # See https://bugs.python.org/issue10272.
    # PyPy3 7.2 has a bug, though: it shares much of the SSL implementation with Python 2,
    # and it unconditionally does `socket.sslerror = SSLError` when ssl is imported.
    # So we can't rely on getattr/hasattr tests, we must be explicit.
    TIMEOUT_ERROR = socket.sslerror if PY2 else socket.timeout # pylint:disable=no-member

    def _setup_listener(self):
        listener, raw_listener = ssl_listener(self.privfile, self.certfile)
        self._close_on_teardown(raw_listener)
        return listener

    def create_connection(self, *args, **kwargs): # pylint:disable=signature-differs
        return self._close_on_teardown(
            # pylint:disable=deprecated-method
            ssl.wrap_socket(super(TestSSL, self).create_connection(*args, **kwargs)))

    # The SSL library can take a long time to buffer the large amount of data we're trying
    # to send, so we can't compare to the timeout values
    _test_sendall_timeout_check_time = False

    # The SSL layer has extra buffering, so test_sendall needs
    # to send a very large amount to make it timeout
    _test_sendall_data = data_sent = b'hello' * 100000000

    test_sendall_array = greentest.skipOnMacOnCI("Sometimes misses data")(
        greentest.skipOnManylinux("Sometimes misses data")(
            test__socket.TestTCP.test_sendall_array
        )
    )

    test_sendall_str = greentest.skipOnMacOnCI("Sometimes misses data")(
        greentest.skipOnManylinux("Sometimes misses data")(
            test__socket.TestTCP.test_sendall_str
        )
    )

    @greentest.skipOnWindows("Not clear why we're skipping")
    def test_ssl_sendall_timeout0(self):
        # Issue #317: SSL_WRITE_PENDING in some corner cases

        server_sock = []
        acceptor = test__socket.Thread(target=lambda: server_sock.append(
            # pylint:disable=no-member
            self.listener.accept()))
        client = self.create_connection()
        client.setblocking(False)
        try:
            # Python 3 raises ssl.SSLWantWriteError; Python 2 simply *hangs*
            # on non-blocking sockets because it's a simple loop around
            # send(). Python 2.6 doesn't have SSLWantWriteError
            expected = getattr(ssl, 'SSLWantWriteError', ssl.SSLError)
            with self.assertRaises(expected):
                client.sendall(self._test_sendall_data)
        finally:
            acceptor.join()
            client.close()
            server_sock[0][0].close()

    # def test_fullduplex(self):
    #     try:
    #         super(TestSSL, self).test_fullduplex()
    #     except LoopExit:
    #         if greentest.LIBUV and greentest.WIN:
    #             # XXX: Unable to duplicate locally
    #             raise greentest.SkipTest("libuv on Windows sometimes raises LoopExit")
    #         raise

    @greentest.ignores_leakcheck
    @greentest.skipOnPy310("No longer raises SSLError")
    def test_empty_send(self):
        # Issue 719
        # Sending empty bytes with the 'send' method raises
        # ssl.SSLEOFError in the stdlib. PyPy 4.0 and CPython 2.6
        # both just raise the superclass, ssl.SSLError.

        # Ignored during leakchecks because the third or fourth iteration of the
        # test hangs on CPython 2/posix for some reason, likely due to
        # the use of _close_on_teardown keeping something alive longer than intended.
        # cf test__makefile_ref
        with self.assertRaises(ssl.SSLError):
            super(TestSSL, self).test_empty_send()

    @greentest.ignores_leakcheck
    def test_sendall_nonblocking(self):
        # Override; doesn't work with SSL sockets.
        pass

    @greentest.ignores_leakcheck
    def test_connect_with_type_flags_ignored(self):
        # Override; doesn't work with SSL sockets.
        pass



if __name__ == '__main__':
    greentest.main()



# Copyright (c) 2018 gevent community
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from . import support

from .sysinfo import PY3
from .sysinfo import PYPY
from .sysinfo import WIN
from .sysinfo import LIBUV

from .sysinfo import EXPECT_POOR_TIMER_RESOLUTION


# Travis is slow and overloaded; Appveyor used to be faster, but
# as of Dec 2015 it's almost always slower and/or has much worse timer
# resolution
CI_TIMEOUT = 15
if (PY3 and PYPY) or (PYPY and WIN and LIBUV):
    # pypy3 is very slow right now,
    # as is PyPy2 on windows (which only has libuv)
    CI_TIMEOUT = 20
if PYPY and LIBUV:
    # slow and flaky timeouts
    LOCAL_TIMEOUT = CI_TIMEOUT
else:
    LOCAL_TIMEOUT = 2

LARGE_TIMEOUT = max(LOCAL_TIMEOUT, CI_TIMEOUT)

# Previously we set this manually to 'localhost'
# and then had some conditions where we changed it to
# 127.0.0.1 (e.g., on Windows or OSX or travis), but Python's test.support says
# # Don't use "localhost", since resolving it uses the DNS under recent
# # Windows versions (see issue #18792).
# and sets it unconditionally to 127.0.0.1.
DEFAULT_LOCAL_HOST_ADDR = support.HOST
DEFAULT_LOCAL_HOST_ADDR6 = support.HOSTv6
# Not all TCP stacks support dual binding where ''
# binds to both v4 and v6.
# XXX: This is badly named; you often want DEFAULT_BIND_ADDR_TUPLE
DEFAULT_BIND_ADDR = support.HOST


DEFAULT_CONNECT_HOST = DEFAULT_CONNECT = DEFAULT_LOCAL_HOST_ADDR
DEFAULT_BIND_ADDR_TUPLE = (DEFAULT_BIND_ADDR, 0)

# For in-process sockets
DEFAULT_SOCKET_TIMEOUT = 0.1 if not EXPECT_POOR_TIMER_RESOLUTION else 2.0

# For cross-process sockets
DEFAULT_XPC_SOCKET_TIMEOUT = 2.0 if not EXPECT_POOR_TIMER_RESOLUTION else 4.0



import unittest
import warnings
import sys

# All supported python versions now provide SSLContext.
# We subclass without importing by name. Compare with
# warning2.py
import ssl

class MySubclass(ssl.SSLContext):
    pass

# This file should only have this one test in it
# because we have to be careful about our imports
# and because we need to be careful about our patching.

class Test(unittest.TestCase):

    @unittest.skipIf(sys.version_info[:2] < (3, 6),
                     "Only on Python 3.6+")
    def test_ssl_subclass_and_module_reference(self):

        from gevent import monkey

        self.assertFalse(monkey.saved)

        with warnings.catch_warnings(record=True) as issued_warnings:
            warnings.simplefilter('always')

            monkey.patch_all()
            monkey.patch_all()

        issued_warnings = [x for x in issued_warnings
                           if isinstance(x.message, monkey.MonkeyPatchWarning)]

        self.assertEqual(1, len(issued_warnings))
        message = str(issued_warnings[0].message)

        self.assertNotIn("Modules that had direct imports", message)
        self.assertIn("Subclasses (NOT patched)", message)
        # the gevent subclasses should not be in here.
        self.assertNotIn('gevent.', message)



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
from gevent.monkey import patch_all
patch_all()


if __name__ == '__main__':
    # Reproducing #728 requires a series of nested
    # imports
    __import__('_imports_imports_at_top_level')



#!/usr/bin/python
# See https://github.com/gevent/gevent/issues/108
import gevent
from gevent import monkey

monkey.patch_all()

import_errors = []


def some_func():
    try:
        from _blocks_at_top_level import x
        assert x == 'done'
    except ImportError as e:
        import_errors.append(e)
        raise

gs = [gevent.spawn(some_func) for i in range(2)]
gevent.joinall(gs)

assert not import_errors, import_errors
